@@ -1,7 +1,7 @@
 #
 #  APRS4R - a ruby based aprs gateway/digipeater
-#  Copyright (C) 2007 by Michael Conrad <do5mc@friggleware.de> and
-#                        Andreas Bier <dl1hrc@web.de>
+#  Copyright (C) 2007 by Michael Conrad <do5mc at aprs4r.org> and
+#                        Andreas Bier <dl1hrc at web.de>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,10 @@ require 'io/nonblock'
 require 'aprs4r/APRS4RLogger'
 
 require 'aprs4r/APRSCall'
+require 'aprs4r/APRSMessage'
+
 require 'aprs4r/Socket'
+require 'aprs4r/SocketError'
 
 
 module APRS4R
@@ -34,14 +37,25 @@ module APRS4R
   class ISSocket < Socket
 
     @logger = APRS4RLogger.get_logger( "ISSocket")
+
+
+    def initialize
+      logger.info( "initialize")
+
+      super
+
+      return
+    end
+
     
-    
-    def initialize( configuration, socket = nil)
-      logger.info( "initialize( configuration | socket)")
+    def setup( configuration, socket = nil)
+      logger.info( "setup( configuration | socket)")
       
       super( configuration)
-      
-      logger.debug( "configuration: #{configuration}")
+
+      @user = "unknown"
+      @auth = false
+
       
       if configuration 
         # socket client mode 
@@ -49,15 +63,21 @@ module APRS4R
         
         @hosts = configuration.hosts
         @port = configuration.port
-        @username = configuration.username
-        @password = ISSocket.generatePassword( @username)
+        @username = configuration.username.upcase if configuration.username
+        @password = ISSocket.generate_password( @username)
         @filter = configuration.filter
         @timeout = configuration.timeout
+
+        if @username.to_s =~ /^MYCALL/i
+          raise "ISSocket: device: #{@name}, invalid call: #{@username}, please use valid call"
+        end
+
       else 
         # socket server mode 
         @enable = true
         @clientMode = false
         @timeout = 0
+        @txEnable = true 
         
         @socket = socket
         @socket.nonblock = true
@@ -66,36 +86,31 @@ module APRS4R
       
       logger.debug( "clientMode: #{@clientMode}")
       
-      # if @enable 
-      #   initISSocket()
-      # end
-      
       return
     end
     
     
-    def initSocket()
-      logger.info( "initSocket()")
+    def init_socket
+      logger.info( "init_socket")
       
       if ! @enable 
         return
       end
       
       if @clientMode
-        initClientSocket
+        init_client_socket
       else 
-        initServerSocket
+        init_server_socket
       end
       
       return
     end
     
     
-    def initClientSocket()
-      logger.info( "initClientSocket()")
+    def init_client_socket
+      logger.info( "init_client_socket")
       
       begin 
-        
         host = @hosts[rand( @hosts.length)]
         
         logger.debug( "connecting to #{host}:#{@port}")
@@ -107,12 +122,14 @@ module APRS4R
         # send login sequence
         username = "user " + @username
         password = "pass " + @password
-        software = "vers aprs4r #{APRS4R::APRS4RVersion}"
+        software = "vers aprs4r #{APRS4RVersion}"
+        software = "vers aprs4r #{APRS4RVersion}-RC-#{APRS4RBuild}" if APRS4RRC == true 
         
-        @socket.puts( username + " " + password + " " + software)
-        @socket.puts( @filter)
+
+        write_data( username + " " + password + " " + software)
+        write_data( @filter)
       rescue Exception => ex
-        logger.warn( "initClientSocket::ex: #{ex}")
+        logger.warn( "init_client_socket: ex: #{ex}")
         
         # wait 5 seconds before next try
         sleep 5
@@ -126,11 +143,15 @@ module APRS4R
     end
     
     
-    def initServerSocket()
-      logger.info( "initServerSocket()")
+    def init_server_socket
+      logger.info( "init_server_socket")
 
-      software = "\# aprs4r #{APRS4R::APRS4RVersion}"
-      @socket.puts( software + 0x0D.chr)
+      software = "\# aprs4r #{APRS4RVersion}"
+      software = "\# aprs4r #{APRS4RVersion}-RC-#{APRS4RBuild}" if APRS4RRC == true
+      write_data( software)
+
+      @user = "unknown"
+      @auth = false
 
       begin
         Timeout::timeout( 5) do
@@ -138,17 +159,36 @@ module APRS4R
           line.chomp! if line
 
           logger.debug( "start: #{line}")
+
+          match_data = line.match( /# +user +([^ ]+) +pass +([^ ]+)/i)
+
+          logger.debug( "match.length: #{match_data.length}")
+          
+          if match_data.length >= 3 
+            user = match_data[1]
+            pass = match_data[2]
+
+            logger.debug( "user: #{user}, pass: #{pass}, generate: #{ISSocket.generate_password( user)}")
+
+            if ISSocket.generate_password( user) == pass
+              @user = user 
+              @auth = true
+            end
+          end
+
         end
       rescue Exception => ex
         logger.warn( "initServerSocket::ex: #{ex}")
       end
 
+      logger.debug( "user: #{@user}, auth: #{@auth}")
+
       return
     end
 
 
-    def reinitSocket()
-      logger.info( "reinitSocket")
+    def reinit_socket
+      logger.info( "reinit_socket")
 
       # close socket (with timeout)
       begin
@@ -158,52 +198,44 @@ module APRS4R
           end
         end
       rescue Exception => ex
-        logger.warn( "reinitSocket::ex: #{ex}")
+        logger.warn( "reinit_socket: #{ex}")
       end
       
       if @clientMode
-        reinitClientSocket()
+        reinit_client_socket
       else 
-        reinitServerSocket()
+        reinit_server_socket
       end
 
       return
     end
     
     
-    def reinitClientSocket()
-      logger.info( "reinitClientSocket")
+    def reinit_client_socket
+      logger.info( "reinit_client_socket")
 
-      initClientSocket()
+      init_client_socket
       
       return
     end
 
 
-    def reinitServerSocket()
-      logger.info( "reinitServerSocket")
+    def reinit_server_socket
+      logger.info( "reinit_server_socket")
 
-      raise "client side down"
+      logger.warn( "server socket reinit")
+
+      # FIXME
+      sleep 10
+
+      raise SocketError, "client side down"
 
       return
     end
 
 
-    def readAPRSMessage
-      logger.info( "readAPRSMessage")
-
-      data = readData
-
-      message = parseAPRSMessage( data)
-
-      logger.debug( "readAPRSMessage: message: #{message} #{message.nil?}")
-
-      return message
-    end
-
-    
-    def readData
-      logger.info( "readData")
+    def read_data
+      logger.info( "read_data")
 
       line = nil
 
@@ -211,25 +243,33 @@ module APRS4R
         Timeout::timeout( @timeout) do
 
           if @socket
-            line = @socket.gets
+            line = @socket.gets.force_encoding( "ASCII-8BIT")
           end
 
-          # if nil, an exception is raised (and reinit is started)
-          line.chomp!
-          logger.debug( "readData::line: -#{line}-")
+          raise SocketError, "nil message" if line.nil?
+
+          line.chomp! 
+          logger.debug( "read_data::line: -#{line}-")
 
         end
+
+      rescue SocketError
+        logger.warn( "read_data: socket ex: #{ex}")
+        raise
+
       rescue Exception => ex
-        logger.warn( "readData::ex: " + ex)
-        reinitSocket()
+        logger.warn( "read_data: ex: #{ex}")
+        # FIXME: insert return nil for benchmark
+        # return nil
+        reinit_socket 
       end until line !~ /^#/
         
-        return line
+      return line
     end
     
 
-    def parseAPRSMessage( data)
-      logger.info( "parseAPRSMessage( data)")
+    def parse_message( data)
+      logger.info( "parse_message( data)")
       
       logger.debug( "data: #{data}")
 
@@ -262,35 +302,23 @@ module APRS4R
         path.delete_at( 0)
         
         path.each{ |entry|
-          call = APRSCall.parse( entry).to_s
-          message.path << call
+          call = APRSCall.parse( entry)
+          message.path << call.to_s if call
         }
 
         # body
         body = data[headerIndex+1...data.length]
         message.payload = body
       end
+
       return message
     end
 
     
-    def writeAPRSMessage( message) 
-      logger.info( "writeAPRSMessage( message)")
-      
-      data = serializeAPRSMessage( message)
+    def serialize_message( message)
+      logger.info( "serialize_message( message)")
 
-      writeData( data)
-      
-      return
-    end
-
-    
-    def serializeAPRSMessage( message)
-      logger.info( "serializeAPRSMessage( message)")
-
-      if message.nil?
-        return nil
-      end
+      return nil if message.nil?
       
       if message.source.nil? || message.destination.nil? 
         logger.warn( "message: NIL")
@@ -299,12 +327,14 @@ module APRS4R
 
       data = String.new
 
-      data = data + message.source.to_s + ">" + message.destination.to_s
+      data += message.source.to_s + ">" + message.destination.to_s
 
       path = message.path
-      
-      for i in 0...path.length
-        data = data + "," + path[i].to_s
+
+      if !path.nil?
+        for i in 0...path.length
+          data += "," + path[i].to_s
+        end
       end
 
       data = data + ":" + message.payload
@@ -313,52 +343,68 @@ module APRS4R
     end
 
 
-    def writeData( data)
-      logger.info( "writeData( data)")
+    def write_data( data)
+      logger.info( "write_wata( data)")
 
-      if data.nil?
-        return
-      end
+      return if data.nil?
 
-      @mutex.synchronize do
-        begin
+      begin
+        @socket_mutex.synchronize do
           if @socket 
-            logger.debug( "writeData::data: -#{data}-")
+            logger.debug( "write_data::data: -#{data}-")
             @socket.puts( data + 0x0D.chr)
             @socket.flush
           end
-        rescue Exception => ex
-          logger.warn( "writeData::ex: " + ex)
-          reinitSocket()
         end
+      rescue Exception => ex
+        logger.warn( "write_data: ex: #{ex}")
+        reinit_socket
+      end
+      
+      return
+    end
+
+
+    def update_filter( filter_calls)
+      logger.info( "update_filter( filter_calls)")
+
+      if !filter_calls.nil? && !filter_calls.empty?
+
+        # build new filter command old filter + (b/call1/call2)
+        command = @filter + " " + "b"
+        filter_calls.each{ |filter|
+          command << "/#{filter.upcase}"
+        }
+        
+        # send new filter command
+        write_data( command)
       end
 
       return
     end
 
     
-    def ISSocket.generatePassword( value)
-      logger.info( "generatePassword( value)")
+    def ISSocket.generate_password( value)
+      logger.info( "generate_password( value)")
 
-      callsign = APRSCall.parse( value)
+      callsign = APRSCall.parse( value.to_s)
       call = callsign.call
 
-      if call.nil?
-        return 
-      end
+      return if call.nil?
 
       # init value
-      value = 0x73E2
+      code = 0x73E2
       i = 0
 
-      while i < call.length
-        value = value ^ (call[i].to_i << 8)
-        value = value ^ (call[i+1].to_i)
+      shift = 8
 
-        i += 2
-      end
+      call.each_byte{ |value|
+        code = code ^ (value << shift)
+        
+        shift = (shift + 8) % 16
+      }
 
-      return (value & 0x7FFF).to_s
+      return (code & 0x7FFF).to_s
     end
 
   end
