@@ -1,6 +1,6 @@
 #
 #  APRS4R - a ruby based aprs gateway/digipeater
-#  Copyright (C) 2007 by Michael Conrad <do5mc@friggleware.de>
+#  Copyright (C) 2008 by Michael Conrad <do5mc@friggleware.de>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 #
 
-require 'aprs4r/ConfigurationAttribute'
 require 'aprs4r/PluginConfiguration'
 
 
@@ -25,26 +24,6 @@ module APRS4R
 
   class StatusPluginConfiguration < PluginConfiguration
 
-    attr_reader :port
-    attr_writer :port
-
-    def initialize( port = nil)
-      @port = port
-    end
-
-    def to_yaml_properties
-      return super + %w{@port}
-    end
-
-    def attributes
-      
-      attributes = [ 
-                    ConfigurationAttribute.new( "port", "number", true, 6, "1234", "TCP-Port")
-                   ]
-
-      return super + attributes
-    end
-
   end
 
 end
