@@ -1,6 +1,6 @@
 #
 #  APRS4R - a ruby based aprs gateway/digipeater
-#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
+#  Copyright (C) 2010 by Michael Conrad <do5mc@aprs4r.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -18,12 +18,30 @@
 #
 
 require 'aprs4r/SocketConfiguration'
+require 'aprs4r/ConfigurationAttribute'
 
 
-class TestSocketConfiguration < SocketConfiguration
+module APRS4R
+
+  class TestSocketConfiguration < SocketConfiguration
+
+    attr_reader :duplicatePeriod
+    attr_writer :duplicatePeriod
+
+
+    def to_yaml_properties
+      super + %w{@duplicatePeriod}
+    end
+
+
+    def attributes
+      attributes = [
+                    ConfigurationAttribute.new( "duplicatePeriod", "number", true, 10, 15,         "Duplikatsinterval")
+                   ]
+
+      return super + attributes 
+    end
 
-  def to_yaml_properties
-    %w{ }
   end
 
 end
