@@ -30,21 +30,44 @@ module APRS4R
     
     # alle units are SI units !!!
     
-    attr_reader :wind_direction, :wind_avg, :wind_gust, :rain_1h, :rain_24h, :rain_day, :temperature, :humidity, :pressure, :luminosity
-    
-    attr_writer :wind_direction, :wind_avg, :wind_gust, :rain_1h, :rain_24h, :rain_day, :temperature, :humidity, :pressure, :luminosity
-    
+    attr_accessor :wind_direction, :wind_avg, :wind_gust, :rain_1h, :rain_24h, :rain_today, :temperature, :humidity, :pressure, :luminosity
 
     @logger = APRS4RLogger.get_logger( "WeatherPlugin")
+
+
+    def initialize
+      logger.info( "initialize")
+
+      super
+
+      return
+    end
+
     
-    
-    def initialize( configuration)
-      logger.info( "initialize( configuration)")
+    def setup( configuration)
+      logger.info( "setup( configuration)")
       
       super( configuration)
+
+      @update_timestamp = nil
       
       return
     end
+
+
+    def update_timestamp
+      @update_timestamp = Time.now
+    end
+
+
+    def is_uptodate?( period)
+
+      return false if @update_timestamp.nil?
+
+      return false if Time.now - @update_timestamp > period
+
+      return true
+    end
     
     
     def WeatherPlugin.celcius_to_fahrenheit( value)
@@ -93,7 +116,25 @@ module APRS4R
       return nil if value.nil? || !value.is_a?( Numeric)
       return value * 0.51444
     end
-    
+
+
+    def WeatherPlugin.knots_to_mph( value)
+      return nil if value.nil? || !value.is_a?( Numeric)
+      return value / 0.8689762
+    end
+
+
+    def WeatherPlugin.mps_to_kph( value) 
+      return nil if value.nil? || !value.is_a?( Numeric)
+      return value * 3.6
+    end
+
+
+    def WeatherPlugin.kph_to_mps( value) 
+      return nil if value.nil? || !value.is_a?( Numeric)
+      return value / 3.6
+    end
+
   end
 
 end
