@@ -25,12 +25,8 @@ module APRS4R
 
   class WeatherPluginConfiguration < PluginConfiguration
 
-    attr_reader :device, :period, :offset, :message
-    attr_writer :device, :period, :offset, :message
-
-
     def to_yaml_properties
-      return super + %w{@device @period @offset @message}
+      return super 
     end
 
   end
